#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <sys/utsname.h>
#include <time.h>
#include <sys/types.h>
#include <unistd.h>

#include "clogger.h"

char clog_program_name[CLOG_MAX_PROGRAM_NAME];
int clog_trace_level = CLOG_TRACE_LEVEL_DEFAULT;

void
clog_set_trace_level(int level)
{
    clog_trace_level = level;
}

int
clog_get_trace_level(void)
{
    return clog_trace_level;
}

void
clog_set_program_name(char *program_name)
{
    strncpy(clog_program_name, program_name, CLOG_MAX_PROGRAM_NAME);
}

/*
 * general trace function for debugging
 */
void
clog_print(const char *file, int line, const char *func,
           const char * fmt, ...)
{
    const int CLOG_MAX_TIME_SIZE = 128;

    struct utsname u;
    time_t t;
    struct tm *cur_tm;
    char curtime[CLOG_MAX_TIME_SIZE];
    va_list args;
    char buf[CLOG_MAX_MESSAGE_SIZE];

    va_start(args, fmt);
    vsprintf(buf, fmt, args);
    va_end(args);

    uname(&u);

    t = time(NULL);
    cur_tm = localtime(&t);
    strftime(curtime, sizeof(curtime), "%b %d %T", cur_tm);

    fprintf(stderr, "%s %s %s[%d]: %s\n", curtime, u.nodename, 
            clog_program_name, (int)getpid(), buf);

    fflush(stderr);
}

void 
clog_error(const char *file, int line, const char *func,
           const char *s)
{
    char buf[CLOG_MAX_MESSAGE_SIZE];

    snprintf(buf, sizeof(buf), "%s at %s:%d, %s", func, file, line, s);
    perror(buf);
}

