#ifndef _CLOGGER_H
#define _CLOGGER_H

/*
 * debugging printf
 */

#define CLOG_MAX_PROGRAM_NAME 256
#define CLOG_MAX_MESSAGE_SIZE 4096

#define CLOG_LEVEL_FATAL   7
#define CLOG_LEVEL_ERROR   6
#define CLOG_LEVEL_WARN    5
#define CLOG_LEVEL_INFO    4
#define CLOG_LEVEL_DEBUG   3
#define CLOG_LEVEL_VERBOSE 2

#define CLOG_TRACE_LEVEL_DEFAULT CLOG_LEVEL_INFO

#define CLOG_TRACE(level, args...)                          \
    if (clog_trace_level <= (level)) {                      \
        clog_print(__FILE__, __LINE__, __FUNCTION__, args); \
    }

#define CLOG_FATAL(args...) CLOG_TRACE(CLOG_LEVEL_FATAL, args)
#define CLOG_ERROR(args...) CLOG_TRACE(CLOG_LEVEL_ERROR, args)
#define CLOG_WARN(args...) CLOG_TRACE(CLOG_LEVEL_WARN, args)
#define CLOG_INFO(args...) CLOG_TRACE(CLOG_LEVEL_INFO, args)
#define CLOG_DEBUG(args...) CLOG_TRACE(CLOG_LEVEL_DEBUG, args)
#define CLOG_VERBOSE(args...) CLOG_TRACE(CLOG_LEVEL_VERBOSE, args)

/* FATAL:   error messages should always be printed and should
 *          occur just before termination.
 *
 * ERROR:   diagnostic error messages, 
 *          always printed to stderr, 
 *          might not result in program termination
 *
 * WARN:    non-fatal error messages, to check, e.g. for weird startup options,
 *          or other conditition that might indicate an error
 *
 * INFO:    program status messages, 
 *          not printed during normal execution,
 *          but can be enabled via command-line argument
 *
 * DEBUG:   debugging messages, might be used around error or other cases
 *          to print additional information
 *
 * VERBOSE: tracing messages, used during program development to dump 
 *          variables, trace control flow,
 *          enabled via command-line arguments
 */

#define CLOG_PERROR(s) clog_error(__FILE__, __LINE__, __PRETTY_FUNCTION__, (s))

extern int clog_trace_level;

void clog_set_program_name(char *program_name);
void clog_set_trace_level(int level);
int clog_get_trace_level(void);
void clog_print(const char *file, int line, const char *func, const char * fmt, ...);
void clog_error(const char *file, int line, const char *func, const char *s);

#endif /* _CLOGGER_H */
